/* 
    Adam Sosnowski - J1TIZ
    A.W.Sosnowski@stud.elka.pw.edu.pl
    Temat: biblioteka stl, opis algorytmu : partial_sum
    File: partial_sum.cpp
*/

/*
    ------- partial_sum -------
    
    Zadaniem algorytmu jest liczenie sum częściowych.
    Innymi słowy liczy on sumę wszystkich elementów poprzedzających
    daną pozycję w ciągu na którym zostanie użyty i wstawia je do ciągu 'result'.
    Występuje on również w drugiej 'wersji', gdzie sumowanie elementów zastępuje się inną,
    wybraną przez programistę operacją.    
*/

/*
    ------- Parametry -------
    
    ----- 'wersja' podstawowa -----
    template <class InputIterator, class OutputIterator>
       OutputIterator partial_sum ( InputIterator first, InputIterator last,
                                    OutputIterator result );
    
    ----- 'wersja' pozwalająca na użycie algorytmu z operacją inną niż sumowanie -----
    template <class InputIterator, class OutputIterator, class BinaryOperation>
       OutputIterator partial_sum ( InputIterator first, InputIterator last,
                                    OutputIterator result, BinaryOperation binary_op );
    
    ----- opis parametrów -----                                
    first, last - iteratory do pierwszego i ostatniego elementu ciągu. Przedział [first, last].
    result -  iterator do ciągu z wynikami działania partial_sum
    binary_op - operacja (obiekt lub funkcja) przyjmująca za argumenty elementy wskazywane przez iteratory z 
              przedziału [first, last] i zwracająca wynik swojego działania (zamiast sumy).
*/

/*
    ------- Wartość zwracana -------
    
    Iterator na koniec stworzonego ciagu.
*/

/* 
    ------- Implementacja algorytmu partial_sum -------
   
    template <class InputIterator, class OutputIterator>
    OutputIterator partial_sum ( InputIterator first, InputIterator last,
                                 OutputIterator result )
    {
      iterator_traits<InputIterator>::value_type val;
      *result++ = val = *first++;
      while (first!=last)
        *result++ = val = val + *first++;
               // or: *result++ = val = binary_op(val,*first++) for binary_op version
      return result;
    }
*/

/*
    ------- Złożoność -------
    Złozoność algorytmu ma charakter liniowy
    Dokładnie jest to: (last - first) - 1 + złożoność binary_op (jeśli jest używany)
*/


/*
    ------- Przykłady -------
    Prezentacja użycia algorytmu partial_sum na wektorach i listach.
*/


#include <iostream>
#include <vector>
#include <functional>
#include <list>

/* Tu znajduje sie m.in partial_sum */
#include <numeric>

using namespace std;


/* Własna, prosta funkcja na potrzeby przykładu */
int moja_funkcja(const int& a, const int& b)
{
    return (a+b+50);
}



int main()
{
   
   vector<int> vec_1(10), vec_2(10);
   vector<int>::iterator vec_iter_1, vec_iter_2, v_iter_end_1, v_iter_end_2;
   
   list <int> li_1;
   list <int>::iterator li_iter, li_iter_end;


   /* Inicjowanie listy kolejnymi liczbami naturalnymi */
   for (int i = 1; i <= 10; ++i)
   {
      li_1.push_back(i);
   }

   /* Wypisywanie zawartości listy */
   cout << "Wartosci kolejnych elementow listy:\n [ " ;
   for (li_iter = li_1.begin(); li_iter != li_1.end(); ++li_iter)
   {
      cout << *li_iter << " ";
   }
   cout << "]" << endl;
   
   
   /* Uzycie pierwszej 'wersji' algorytmu partial_sum.
      Elementy z listy sumowane są do wektora */
   v_iter_end_1 = partial_sum (li_1.begin (), li_1.end (), vec_1.begin());
   
   cout << "Zawartosc wektora (ciagu) po uzyciu algorytmu partial_sum:\n [ " ;
   for ( vec_iter_1 = vec_1.begin(); vec_iter_1 != v_iter_end_1; ++vec_iter_1)
   {
      cout << *vec_iter_1 << " ";
   }
   cout << "]" << endl;

   /* Uzycie drugiej 'wersji' algorytmu partial_sum.
      Używamy funkcji multiplies.
      Dane wejściowe: elementy z listy.
      Dane wyjściowe: wektor. */
   v_iter_end_2 = partial_sum (li_1.begin(), li_1.end(), vec_2.begin(), multiplies<int>());
   
   cout << "Zawartosc wektora (ciagu) po uzyciu algorytmu partial_sum w drugiej 'wersji'\n"
            << "(Uzyto funkcji multiplies): \n [ " ;
   for ( vec_iter_2 = vec_2.begin(); vec_iter_2 != v_iter_end_2 ; ++vec_iter_2)
   {
      cout << *vec_iter_2 << " ";
   }
   cout << "]" << endl;

   /* Uzycie drugiej 'wersji' algorytmu partial_sum.
      Używamy własnej funkcji.
      Dane wejściowe: elementy z listy.
      Dane wyjściowe: lista. */
   li_iter_end = partial_sum (li_1.begin(), li_1.end (), li_1.begin(), moja_funkcja);
   cout << "Zawartosc listy (ciagu) po uzyciu algorytmu partial_sum w drugiej 'wersji'\n"
            << "(Uzyto wlasnej funkcji): \n [ " ;
   for (li_iter = li_1.begin(); li_iter != li_iter_end; ++li_iter)
   {
      cout << *li_iter << " ";
   }
   cout << "]" << endl;
   
   
   return 0;
   
}




